// Constants
const float THROW_BOUNCE_SPEED = 150.0f;
const float THROW_BOUNCE_MARGIN = 0.0f;
const float THROW_BOUNCE_GRAVITY = 0.15f;

// Called at the start of the page
void InitPickUpAndThrow(Object@ obj)
{
	// Set initial velocity,rotation
	obj.$velocity = Vector2(randf()*6.0f - 3.0f,randf()*2.0f - 1.0f);	
	
	// Adjust pivot point
	obj.pivot = obj.size * 0.5f;
	obj.position = obj.position + obj.size * 0.5f * obj.scale;
}


event PickUpAndThrow.OnMouseClick
{
	if (mouseButton == MB_LEFT)
	{
		// Begin dragging
		self.$Drag = true;	
		self.$OrigPos = self.position;	
		self.$MousePos = GetMousePosition();
	}
}

event PickUpAndThrow.OnMouseRelease
{
	if (mouseButton == MB_LEFT && self.$Drag == true)
	{
		// Stop dragging
		self.$Drag = false;
	}
}

event PickUpAndThrow.OnUpdate
{
	// Is dragging?
	if (self.$Drag == true)
	{
		// Drag
		Vector2 DeltaMove = GetMousePosition() - self.$MousePos.AsVector2();
		
		Vector2 PrevPos = self.position;
		self.position = self.$OrigPos.AsVector2() + DeltaMove;
		
		self.$velocity = (self.position - PrevPos) * 0.5f + Vector2(-1.0f + randf()*2.0f, -1.0f + randf()*2.0f);
		self.$resting = false;
	}
	else
	{
		// Bounce around
		if (_VAR_EXISTS(self,"velocity") == false)
			InitPickUpAndThrow(self);
		
		// Get velocity, current position
		Vector2 velocity = self.$velocity.AsVector2();
		Vector2 currPos = self.position - (self.pivot * self.scale);

		// Apply gravity
		velocity.y += THROW_BOUNCE_GRAVITY * elapsed * THROW_BOUNCE_SPEED;

		// Check for horizontal bounce
		if ((velocity.x < 0 && currPos.x < 0.0f) || 
		(velocity.x > 0 && currPos.x + self.size.x * self.scale * (1.0f-THROW_BOUNCE_MARGIN) > 800)		)
		{
			velocity.x *= -1.0f;		// Invert horizontal velocity
		}

		// Check for vertical bounce
		if (velocity.y > 0 && currPos.y + self.size.y * self.scale * (1.0f-THROW_BOUNCE_MARGIN) > 600)
		{
			self.position.y -= 1.0f;
			velocity.y *= -0.8f;		// Invert vertical velocity, lose energy
			if (abs(velocity.y) < 0.5f)	// If the bounce is small enough..
			{
				if (abs(velocity.x) > 2.0f)
				{
					// steal some from horizontal
					velocity.y += 2.0f;
					velocity.x -= (velocity.x > 0) ? 0.5f : -0.5f;
				}
				else
					self.$resting = true;
			}
							
			velocity.x *= 0.75f;
			if (abs(velocity.x) < 0.01f)
				velocity.x = 0.0f;
		}
		
		// Update new position, rotation and velocity
		if (self.$resting == false)
		{
			self.position = self.position + velocity * elapsed * THROW_BOUNCE_SPEED;
			self.$velocity = velocity;		
			
		}
	}
	
}